/*
 * main.c
 *	Description: Aplicacio interpret del protocol CSE.
 *	Llegeix solicituds per la UART3 i les envia al interpret per a procesarles.
 *  Created on: 21/3/2015
 *      Author: David Otal
 */

#include "Interpret.h"
#include "FreeRTOS.h"
#include "task.h"
#include "LPC17xx.h"
#include "type.h"
#include "uart.h"
#include "queue.h"
#include "stdlib.h"
#include "ConfigCSE.h"
#include "Log.h"
/*#include "basic_io.h" */



/* Used as a loop counter to create a very crude delay. */
#define mainDELAY_LOOP_COUNT		( 0xfffff )
extern volatile uint32_t UART3Count;
extern volatile uint8_t UART3Buffer[BUFSIZE];

/* Tasca principal. Llegeix dades de la UART3 i les envia al
 * interpret del protocol.
 * @precondition 	Entradas per UART3
 * 					Velocitad de transmissio 9.600
 * @param pvParameters	Paràmetres de la tasca.
 *
 * @return void
 */
static void vScanTask( void *pvParameters );

/*-----------------------------------------------------------*/
/* Declare a variable of type xQueueHandle.  This is used to store the queue
that is accessed by all three tasks. */
xQueueHandle xQueue;
/*-----------------------------------------------------------*/

int main( void )
{	PrintF_start( OUTPUT_PORT, OUTPUT_SPEED);
	Log_start(LOG_PORT, LOG_SPEED);

   	/* Creacio de la tasca principal de control.
   	 * Per defecte prioritat 1  */
   	xTaskCreate( vScanTask, "interpret", 240, ( void * ) 100, 1, NULL );
   	/* Start the scheduler so the created tasks start executing. */
   	vTaskStartScheduler();

	/* If all is well we will never reach here as the scheduler will now be
	running.  If we do reach here then it is likely that there was insufficient
	heap available for the idle task to be created. */
	for( ;; );
	return 0;
}


static void vScanTask( void *pvParameters )
{	long lValueToSend;
	portBASE_TYPE xStatus;
	char bufferScan[255] = "\0";
	char * pcTaskscan = bufferScan;
	int size;

	/* As per most tasks, this task is implemented within an infinite loop. */
	for( ;; )
	{
		pcTaskscan = "/0";
		size = PrintF_scan(3, bufferScan, 10);
		if (size > 0) {
			Log_log ("new %i input bytes detected", size );
			Interpret_run(bufferScan);
		}
		/* Allow the other sender task to execute. */
		taskYIELD();
	}
}


/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}


