/*
 * ledControl.c
 *
 *  Created on: 21/5/2015
 *      Author: david
 *	Control de Led de la placa LPC1769
 */
/*-----------------------------------------------------------*/

#include "LPC17xx.h"
#include "FreeRTOS.h"
#include "timers.h"
#include "task.h"
#include "type.h"
#include "ProtocolSE.h"
#include "led.h"


int countParmLed = 0;
unsigned int timeLed = 0;
xTaskHandle xHandle = 0;


/* This function configures a timer to be used as the run time statistics time
base.  The function implementation requires knowledge of the LPC17xx peripheral
registers, and uses macros that are available when the LPC17xx.h header file is
used.  In this example LPC17xx.h is included from FreeRTOSConfig.h. */
void _vSetupTimerForLedTimeStats( void )
{
const unsigned long TCR_COUNT_RESET = 2, CTCR_CTM_TIMER = 0x00, TCR_COUNT_ENABLE = 0x01;

	/* Power up and drive the timer 0. */
	LPC_SC->PCONP |= 0x02UL;
	LPC_SC->PCLKSEL0 = ( LPC_SC->PCLKSEL0 & (~(0x3<<2)) ) | ( 0x01 << 2 );

	/* Reset Timer 0 */
	LPC_TIM0->TCR = TCR_COUNT_RESET;

	/* The timer needs to just count up continuously. */
	LPC_TIM0->CTCR = CTCR_CTM_TIMER;

	/* The clock driving the timer is prescaled to a frequency that is good enough
	to get a decent resolution,	but not so fast that the counter value will
	overflow too quickly. */
	LPC_TIM0->PR =  ( configCPU_CLOCK_HZ / 10000UL ) - 1UL;

	/* Start the counter. */
	LPC_TIM0->TCR = TCR_COUNT_ENABLE;
}

static void _vLedTask( void *pvParameters )
{   int i;
	volatile unsigned long ulLoopCounter;
	const unsigned long ulMaxLoopCount = 0x1fffUL;
	portTickType xLastExecutionTime;

	/* The task will run every time milliseconds. */
	const portTickType xBlockPeriod = ( timeLed / portTICK_RATE_MS );

	/* Initialise xLastExecutionTime to the current time.  This is the only
	time this variable needs to be written to explicitly.  Afterwards it is
	updated internally within the xTaskDelayUnitl() API function. */
	xLastExecutionTime = xTaskGetTickCount();

	/* As per most tasks, this task is implemented in an infinite loop. */
	for( ;; )
	{
		led_toggle();
		/* Wait until it is time to run this task again. */
		vTaskDelay( xBlockPeriod );
	}
}

void LedControl_set(unsigned int * ptParm, unsigned char size){
	countParmLed++;
	if (countParmLed == 1){
		timeLed = *ptParm;
	}
}

void LedControl_run(unsigned short int action){
	if (xHandle != 0) {
		vTaskDelete( xHandle );
	}
	led_init();
	if ( action == ID_ON ) {
		led_on();
		//Log_log("LED ON");
	}
	else if ( action == ID_OFF ) {
		led_off();
		//Log_log("LED OFF");
	}
	else if ( action == ID_BLINK ) {
		//Log_log("LED BLINK");
		if (timeLed == 0) led_toggle();
		else {
			_vSetupTimerForLedTimeStats();
			/* prioritat 2 */
			xTaskCreate( _vLedTask, "blinkLed", 240, NULL,	2, &xHandle );
		}
	}
	else Log_log("Accio incorrecta en LED");
	countParmLed = 0;
}



