/*
 * adcControl.c
 *
 *  Created on: 27/5/2015
 *      Author: david
 */

#include "type.h"
#include "LPC17xx.h"
#include "FreeRTOS.h"
#include "timers.h"
#include "task.h"
#include "adc.h"
#include "ProtocolSE.h"
#include "adcControl.h"
#include "uart.h"
#include "Log.h"
#include "PrintF.h"

extern volatile uint32_t ADCValue[ADC_NUM];
extern volatile uint32_t ADCIntDone;

int ADC_countParm = 0;
int num_ports = 0;
unsigned int ADC_time = 0;
unsigned char port[6];
xTaskHandle adcxHandle = 0;


/* This function configures a timer to be used as the run time statistics time
base.  The function implementation requires knowledge of the LPC17xx peripheral
registers, and uses macros that are available when the LPC17xx.h header file is
used.  In this example LPC17xx.h is included from FreeRTOSConfig.h. */
void _vSetupTimerForADCTimeStats( void )
{
const unsigned long TCR_COUNT_RESET = 2, CTCR_CTM_TIMER = 0x00, TCR_COUNT_ENABLE = 0x01;

	/* Power up and drive the timer 0. */
	LPC_SC->PCONP |= 0x02UL;
	LPC_SC->PCLKSEL0 = ( LPC_SC->PCLKSEL0 & (~(0x3<<2)) ) | ( 0x01 << 2 );

	/* Reset Timer 0 */
	LPC_TIM0->TCR = TCR_COUNT_RESET;

	/* The timer needs to just count up continuously. */
	LPC_TIM0->CTCR = CTCR_CTM_TIMER;

	/* The clock driving the timer is prescaled to a frequency that is good enough
	to get a decent resolution,	but not so fast that the counter value will
	overflow too quickly. */
	LPC_TIM0->PR =  ( configCPU_CLOCK_HZ / 10000UL ) - 1UL;

	/* Start the counter. */
	LPC_TIM0->TCR = TCR_COUNT_ENABLE;
}

static void _vadcTask( void *pvParameters ) {
    int i;
	volatile unsigned long ulLoopCounter;
	const unsigned long ulMaxLoopCount = 0x1fffUL;
	portTickType xLastExecutionTime;

	/* The task will run every time milliseconds. */
	const portTickType xBlockPeriod = ( ADC_time / portTICK_RATE_MS );

	/* Initialise xLastExecutionTime to the current time.  This is the only
	time this variable needs to be written to explicitly.  Afterwards it is
	updated internally within the xTaskDelayUnitl() API function. */
	xLastExecutionTime = xTaskGetTickCount();

	/* As per most tasks, this task is implemented in an infinite loop. */
	for(;;)
	{
		adcControl_get();
		generate_paquetSend();
		/* Wait until it is time to run this task again. */
		vTaskDelay( xBlockPeriod );
	}
}


void adcControl_get(){
	uint32_t adcData = 0;
	float    fData = 0;
	int i;
	for( i=0; i<num_ports;i++){
		adcData = ADCRead( port[num_ports] );
		/* Conversio de ticks a valor */
		fData = (adcData * ADC_VOLTS ) / ADC_REF_NUM_BITS;
		/* Retorn del valor * 1000 com a enter */
		adcData = (int) (fData * 1000 );
		generate_Integer(ID_ADC, adcData);
	}
}

void adcControl_set(unsigned char * ptParm, unsigned char size){
	unsigned int * time;
	 if (adcxHandle != 0) {
		//	vTaskDelete( adcxHandle );
		//	adcxHandle = 0;
		vTaskSuspend( adcxHandle );
	}
	ADC_countParm++;
	if (ADC_countParm == 1) {
		time = (unsigned int *) ptParm;
		ADC_time = *time;
		num_ports = 0;
	}
	else if (ADC_countParm > 1) {
		port[num_ports] = *ptParm;
		num_ports++;
	}
}

void adcControl_run(unsigned short int action){
	uint32_t velocitat = DEFAULT_SPEED;
	ADCInit(velocitat);
	//Log_log ("Funcio ADC accio=%s ", action );
	if ( action == ID_GET ) {
		if (ADC_time == 0 )
			adcControl_get();
		else {
			_vSetupTimerForADCTimeStats();
			/* prioritat 2 */
			if (adcxHandle == 0) {
				xTaskCreate( _vadcTask, "readingADC", 240,( void * ) 100, 2, &adcxHandle );
			}
			else
				vTaskResume( &adcxHandle );
		}

	}
	else PrintF_print (3, "error action" );
	ADC_countParm=0;
}
