/**
 * Description: Implementació de entrades i sortides del port serie,
 * fent servir UART3
 *
 * author: David Otal Esclarin
 *
 * revision 1.0 date 26/03/2015
 */

#ifndef LIBUOC_INC_PRINTF_H
#define LIBUOC_INC_PRINTF_H_
/* Definicio dels ports */
#define UART0port	(0)
#define UART1port	(1)
#define UART3port	(3)

/* Definicions per als interruptors dels semafors */
#define mainSW_INTERRUPT_ID		( 0 )	/* id. interrupcio */
#define mainTRIGGER_INTERRUPT()	NVIC_SetPendingIRQ( mainSW_INTERRUPT_ID )
#define mainCLEAR_INTERRUPT()	NVIC_ClearPendingIRQ( mainSW_INTERRUPT_ID )
#define mainSOFTWARE_INTERRUPT_PRIORITY 		( 5 )

/* Referencies externes */
extern volatile uint8_t UART0Buffer[BUFSIZE];
extern volatile uint8_t UART1Buffer[BUFSIZE];
extern volatile uint8_t UART3Buffer[BUFSIZE];

/* Inicialització de la UART
 * @precondition 	Cap
 *
 * @param portNum	Port de la UART ( 0, 1 o 3 )
 * @param baudrate	Velocitat de transmissió en baudis
 *
 * @return void
 */
void PrintF_start(uint32_t portNum, uint32_t baudrate);

/* Envia un missatge de text a traves de la UART. El fi del missatge
 * es el caracrter 0, com a tots els Strings de C.
 * Format semblant al printf(), en que es poden envia rvariables.
 * @precondition 	Haver fet un PrintF_start() previament
 *
 * @param portNum	Port de sortida de la UART
 * @param format	Missatge a enviar. Mateix format que el printf(). Multivariable.
 *
 * @return void
 */
void PrintF_print(uint8_t port, const char * format, ... );

/* Envia una seqüencia de dades a traves de la UART. No accepta enviament de variables.
 *
 * @precondition 	Haver fet un PrintF_start() previament
 *
 * @param portNum	Port de sortida de la UART
 * @param stream	Seqüencia de caracters a enviar.
 * @param size		Mida de la seqüencia de dades a enviar.
 *
 * @return void
 */
void PrintF_stream(uint8_t port, char * stream, int size );

/* Captura de texte del port serie, via UART
 * @precondition 	Haver fet un PrintF_start() previament
 *
 * @param portNum	Port de entrada de la UART
 * @param timeout	Temps de espere màxim
 *
 * @return 			Retorna la longitud del texte llegit
 */
int PrintF_scan(uint8_t port, char * text, uint32_t timeout);

#endif /* LIBUOC_INC_PRINTF_H */
