'''
Created on 01/05/2015

@author: david
'''

from pyCSE import defSE
from pyCSE import serialCon
from pyCSE import log
from pyCSE import protocolCSE
from pyCSE import scanSE
from pyCSE import led
from pyCSE import adc
from pyCSE import uart
from pyCSE import i2c
from pyCSE import spi
from pyCSE import pin

class PyCSE(object):
    '''
    classdocs
    '''
    logCSE = log.Log()
    
    def __init__(self):
        '''
        Constructor        
        Declara totes les interficies de treball amb la API
        '''        
        self.prot = protocolCSE.Protocol()
        self.myled = led.Led()
        self.myPin = pin.Pin()
        self.myADC = adc.ADC()
        self.myUART = uart.UART()
        self.mySPI = spi.SPI()
        self.myI2C = i2c.I2C()        

    def serialConnection(self):
        '''
        Conexió amb la placa de control 1769 via Port serie
        '''
        self.con = serialCon.SerialCon()
        self.prot.reset()        
        return (self.con.isOpen())
        
    def showLog(self):
        ''' Visualització del Log '''
        self.logCSE.show()
        
    def sendAndWait(self, paquet):
        ''' Envia un paquet i espera resposta '''
        dades = []
        stream = paquet.toBytes()
        self.con.send(stream)
        tramaID = paquet.getTramaID()
        paquetList = self.getResposta(tramaID)
        for paquet in paquetList:
            print ("paquet found")
            myScaner = scanSE.Scaner(paquet)
            dades = myScaner.getDades() 
        return dades   
           
    def ledOn(self):
        ''' Encen el Led de la placa '''                
        self.myled.on()
        paquet = self.prot.newPaquet(self.myled)
        dades = self.sendAndWait(paquet)                
        return dades
    
    def ledOff(self):
        ''' Apaga el Led de la placa '''
        self.myled.off()
        paquet = self.prot.newPaquet(self.myled)
        dades = self.sendAndWait(paquet)                 
        return dades
    
    def ledBlink(self, time):
        ''' Intermitents amb el Led de la placa '''
        self.myled.blink(time)
        paquet = self.prot.newPaquet(self.myled)        
        dades = self.sendAndWait(paquet)
        return dades

    def pinGet(self, pin):
        ''' Llegeix l'estat d'un pin '''
        self.myPin.pinGet(pin)
        paquet = self.prot.newPaquet(self.myPin)
        dades = self.sendAndWait(paquet)        
        return dades        

    def pinSet(self, pin, value):
        ''' Determina l'estat d'un pin '''
        self.myPin.pinSet(pin, value)
        paquet = self.prot.newPaquet(self.myPin)
        dades = self.sendAndWait(paquet)        
        return dades        
                
    def getADC(self, mstime, listPorts):
        ''' Llegeix dades del conersor analogic - digital '''
        dades = []
        self.myADC.get(mstime, listPorts)
        paquet = self.prot.newPaquet(self.myADC)        
        stream = paquet.toBytes()
        self.con.send(stream)
        tramaID = paquet.getTramaID()
        if mstime == 0:
            paquetList = self.getResposta(tramaID)    
            for elemento in paquetList:                                    
                myScaner = scanSE.Scaner(elemento)
                dades = myScaner.getDades()
        else:
            dades = tramaID            
        return dades;
    
    def getUART(self, port, speed):
        ''' Obtenció de dades de la unitat UART '''
        dades = []
        self.myUART.get(port, speed)
        paquet = self.prot.newPaquet(self.myUART)
        stream = paquet.toBytes()
        self.con.send(stream)
        tramaID = paquet.getTramaID()
        paquetList = self.getResposta(tramaID)
        for element in paquetList:
            myScaner = scanSE.Scaner(element)
            dades = myScaner.getDades()                    
        return dades;
    
    def sendUART(self, port, speed, text):
        ''' Enviament de dades a la unitat UART '''
        dades = []
        self.myUART.send(port, speed, text)
        paquet = self.prot.newPaquet(self.myUART)        
        stream = paquet.toBytes()
        self.con.send(stream)
        tramaID = paquet.getTramaID()
        paquetList = self.getResposta(tramaID)
        for element in paquetList:
            myScaner = scanSE.Scaner(element)
            dades = myScaner.getDades()                    
        return dades;
    
    def getSPI(self, port):
        ''' Obtenció de dades de la unitat SPI '''
        dades = []
        self.mySPI.get(port)
        paquet = self.prot.newPaquet(self.mySPI)
        stream = paquet.toBytes()
        self.con.send(stream)
        tramaID = paquet.getTramaID()
        paquetList = self.getResposta(tramaID)
        for element in paquetList:
            myScaner = scanSE.Scaner(element)
            dades = myScaner.getDades()                    
        return dades;
    
    def sendSPI(self, port, text):
        ''' Enviament de dades a la unitat SPI '''
        dades = []
        self.mySPI.send(port, text)
        paquet = self.prot.newPaquet(self.mySPI)        
        stream = paquet.toBytes()
        self.con.send(stream)
        tramaID = paquet.getTramaID()
        paquetList = self.getResposta(tramaID)
        for element in paquetList:
            myScaner = scanSE.Scaner(element)
            dades = myScaner.getDades()                    
        return dades
    
    def getI2C(self):
        self.myI2C.get()
        paquet = self.prot.newPaquet(self.myI2C)
        stream = paquet.toBytes()
        self.con.send(stream)
        tramaID = paquet.getTramaID()
        paquet = self.getResposta(tramaID)
        myScaner = scanSE.Scaner(paquet)
        dades = myScaner.getDades()            
        return dades;
    
    def findPaquets(self, stream, tramaID, paquetList):
        ''' Busca paquets de resposta, en una seqüencia de dades '''
        sortir = False
        ini = 0
        fin = len(stream)
        index = 0                            
        while not sortir:
            index = stream.find(tramaID, ini, fin)
            if index < 0 or index > defSE.val.READ_SIZE:
                sortir = True
            else:                
                paquet = self.prot.loadPaquet(stream[index:fin])                
                ini = index + paquet.size()
                paquetList.append(paquet)                    
        return paquetList            

    def findLogs(self, stream):
        ''' Busca missatges del Log del firmware, en una seqüencia de dades '''
        sortir = False
        ini = 0
        fin = len(stream)        
        index = 0        
        indexFi = 0                    
        while not sortir:
            index = stream.find(b"fwLog", ini, fin)
            if index < 0 or index > defSE.val.READ_SIZE:
                sortir = True
            else:
                indexFi =  stream.find( 0, index, fin)
                self.logCSE.addInfo( stream[index:indexFi] )                
                ini = indexFi
        return 
    
                        
    def getResposta(self, tramaID): 
        ''' Busca la resposta a una solicitud, en una seqüencia de dades '''   
        paquetList = []        
        dades = self.con.read()
        while len(dades) > 0:
            nextDades = self.con.read()
            dades = dades + nextDades                          
            self.findPaquets(dades, tramaID, paquetList)
            self.findLogs(dades)
            dades = nextDades                       
        return(paquetList)         
        
        
        